#!/usr/bin/python
#
# Read one or more output directories and create a single stop list.
# Outputs as both a stop list and as a cdb data file

import sys
if sys.version_info < (3,1):
    raise RuntimeError("idiffernece.py now requires Python 3.1 or above")

import glob,re,os
from subprocess import call,Popen,PIPE

feature_files = ['ccn.txt','domain.txt','email.txt','exif.txt', 'rfc822.txt','telephone.txt','url.txt','zip.txt']

def create_stop_list(featuredir,ofn):
    global options

    def process_file(fname):
        print("processing",fname)
        for line in open(fname,'rb'):
            if line[0]==b'#': continue
            if line[0:2]==b'n=': continue
            try:
                (offset,word,context) = line.split(b'\t')
                stop_list[context.strip()] = fname
            except ValueError:
                pass

    stop_list = {}
    for (dirpath,dirnames,filenames) in os.walk(featuredir):
        for filename in filenames:
            if filename in feature_files:
                process_file(os.path.join(dirpath,filename))

    f = open(ofn,"wb")
    for key in sorted(stop_list.keys()):
        f.write(key)
        f.write(b'\t')
        f.write(stop_list[key].encode('utf-8'))
        f.write(b'\n')

if __name__=="__main__":
    from optparse import OptionParser
    global options

    parser = OptionParser()
    parser.add_option("--wipe",action="store_true",default=False)
    parser.usage = """"usage: %prog [options] output-file.txt\n
Either process the virtual machines in [vmdir] or recursively read the feature files in [featuredir]
and write to <output-file> a context stop list for use with bulk_extractor."""
    parser.add_option("--vmdir",help="specifies a directory of virtual machines on which bulk_extractor should be run")
    parser.add_option("--featuredir",help="specifies the directory of output files for bulk_extractor and input dirs this program")
    parser.add_option("--bulk_extractor",help="specifies bulk_extractor executable to use",default="bulk_extractor")
    (options,args) = parser.parse_args()

    if len(args)==0:
        parser.print_help()
        exit(0)

    if options.vmdir:
        print("Running Bulk Extractor...")
        vms = glob.glob(options.vmdir + "/*/*-flat.vmdk")
        os.mkdir(options.featuredir)
        for vm in vms:
            outname = os.path.join(options.featuredir,re.search("/([^/]*)-(tpl|flat)",vm).group(1))
            cmd = [options.bulk_extractor,'-o',outname,vm]
            print("\n\n\n")
            print("Image {0} of {1}".format(vms.index(vm),len(vm)))
            print(" ".join(cmd))
            call(cmd)

    print("Creating stop list from",options.featuredir)
    create_stop_list(options.featuredir,args[0])
    exit(0)
                                          
    



    




