#!/usr/bin/python
#
# postprocess the EXIF output file into a CSV
#

import xml.parsers.expat,os,os.path,sys

class ExifParser:
    def __init__(self,data):
        self.data = {}
        p = xml.parsers.expat.ParserCreate()
        p.StartElementHandler = self.start_element
        p.CharacterDataHandler = self.char_data
        p.Parse(data)
    def start_element(self,name,attrs):
        self.element = name
        if self.element not in self.data:
            self.data[self.element] = ""
    def char_data(self,data):
        self.data[self.element] += data

if __name__=="__main__":
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option("--zap",help="erase outfile",action="store_true")
    parser.usage = "usage: %prog [options] input.txt output.csv"
    (options,args) = parser.parse_args()
    if len(args)!=2:
        parser.print_help()
        exit(1)

    (infile,outfile) = args
    
    if os.path.exists(outfile) and not options.zap:
        print outfile,"Exists"
        exit(1)

    invalid_tags = 0
    tags = set()
    print "Input file:",infile
    print "Output file:",outfile
    out = open(outfile,"w")
    print "Scanning for EXIF tags..."
    for line in open(infile):
        if line[0:1]=='#': continue
        (offset,hash,xmlstr) = line.split("\t")
        try:
            p = ExifParser(xmlstr)
            for tag in p.data.keys():
                tags.add(tag)
        except xml.parsers.expat.ExpatError:
            invalid_tags += 1
            pass
    taglist = list(tags)
    print "There are %d exif tags " % (len(taglist))
    if invalid_tags: print "There are %d invalid tags" % invalid_tags

    import csv
    writer = csv.writer(open(outfile,"w"),delimiter=',',quotechar='"',quoting=csv.QUOTE_MINIMAL)
    writer.writerow(taglist)
    for line in open(infile):
        (offset,hash,xmlstr) = line.split("\t")
        try:
            p = ExifParser(xmlstr)
            writer.writerow([p.data.get(key,"") for key in taglist])
        except xml.parsers.expat.ExpatError:
            print "Invalid XML:",xmlstr
            pass




        
        
