/**
 * histogram.cpp:
 */

#include "bulk_extractor.h"
#include "histogram.h"
#include "utf8.h"

using namespace std;

ostream & operator << (ostream &os, const HistogramMaker::FrequencyReportVector &rep){
    for(HistogramMaker::FrequencyReportVector::const_iterator i = rep.begin(); i!=rep.end();i++){
	os << "n=" << i->tally.count << "\t" << i->value;
	if(i->tally.count16>0) os << "\t(utf16=" << i->tally.count16<<")";
	os << "\n";
    }
    return os;
}

HistogramMaker::FrequencyReportVector *HistogramMaker::makeReport()  const
{
    FrequencyReportVector *rep = new FrequencyReportVector();
    for(HistogramMap::const_iterator it = h.begin(); it != h.end(); it++){
	rep->push_back(ReportElement(it->first,it->second));
    }
    sort(rep->begin(),rep->end(),ReportElement::compare);
    return rep;
}

HistogramMaker::FrequencyReportVector *HistogramMaker::makeReport(int topN) const
{
    HistogramMaker::FrequencyReportVector   *r2 = makeReport();	// gets a new report
    HistogramMaker::FrequencyReportVector::iterator i = r2->begin();
    while(topN>0 && i!=r2->end()){	// iterate through the first set
	i++;
	topN--;
    }
    r2->erase(i,r2->end());
    return r2;
}

bool HistogramMaker::looks_like_utf16(const std::string &str,bool &little_endian)
{
    if((uint8_t)str[0]==0xff && (uint8_t)str[1]==0xfe){
	little_endian = true;
	return true; // begins with FFFE
    }
    if((uint8_t)str[0]==0xfe && (uint8_t)str[1]==0xff){
	little_endian = false;
	return true; // begins with FFFE
    }
    /* If none of the even characters are NULL and some of the odd characters are NULL, it's UTF-16 */
    uint32_t even_null_count = 0;
    uint32_t odd_null_count = 0;
    for(size_t i=0;i+1<str.size();i+=2){
	if(str[i]==0) even_null_count++;
	if(str[i+1]==0) odd_null_count++;
    }
    if(even_null_count==0 && odd_null_count>1){
	little_endian = true;
	return true;
    }
    if(odd_null_count==0 && even_null_count>1){
	little_endian = false;
	return true;
    }
    return false;
}

void HistogramMaker::add(const std::string &key)
{
    if(key.size()==0) return;		// don't deal with zero-length keys

    const std::string *keyToAdd = &key;
    std::string *key8 = 0;		// place to hold UTF8 key
    bool little_endian;
    if(looks_like_utf16(*keyToAdd,little_endian)){
	/* re-image this string as UTF16*/
	std::wstring utf16;
	for(size_t i=0;i<key.size();i+=2){
	    if(little_endian) utf16.push_back(key[i] | (key[i+1]<<8));
	    else utf16.push_back(key[i]<<8 | (key[i+1]));
	}
	/* Now convert it to a UTF-8;
	 * set key8 to be the utf-8 string that iwll be erased.
	 */
	key8 = new std::string;
	try {
	    utf8::utf16to8(utf16.begin(),utf16.end(),std::back_inserter(*key8));
	    /* Erase any nulls if present */
	    while(key8->size()>0) {
		size_t nullpos = key8->find('\000');
		if(nullpos==string::npos) break;
		key8->erase(nullpos,1);
	    } 
	    keyToAdd = key8;
	} catch(utf8::invalid_utf16){
	    /* Exception; bad UTF16 encoding */
	    delete key8;
	    key8 = 0;
	}
    }
    if(debug & DEBUG_MALLOC_FAIL){
	if((h.size() % MALLOC_FAIL_FREQUENCY)==(MALLOC_FAIL_FREQUENCY-1)){
	    throw bad_alloc();
	}
    }

    h[*keyToAdd].count++;
    if(key8){
	h[*keyToAdd].count16++;
	delete key8;
    }
}
    
