/**
 * a linelist_file reads all of the lines in a file and then says
 * if a queryed string is in the file or not.
 * 
 * Lines can contain wildcards such as ? or *. If so, these are turned into 
 * the regular expressions "." and ".*"
 */

#ifndef LINELIST_FILE_H
#define LINELIST_FILE_H

#include <fstream>
#include <string>
#include <set>
#include <vector>

#include "myregex.h"

using namespace std;

/**
 * The regex_list maintains a list of literal strings and regular expressions.
 * The list can be read out of a file.
 * check() returns true if the provided string is inside the list
 */
class regex_list {
 public:
    set<string>literal_strings;			/* static strings */
    vector<myregex *> patterns;
    regex_list():literal_strings(),patterns(){}

    size_t size(){
	return literal_strings.size() + patterns.size();
    }
    /**
     * Read a file; returns 0 if successful, -1 if failure.
     * @param fname - the file to read.
     */
    ~regex_list(){
	for(vector<myregex *>::iterator it=patterns.begin(); it != patterns.end(); it++){
	    delete *it;
	}
    }
    void add_regex(const string &pat){
	patterns.push_back(new myregex(pat,0));
    }
    int readfile(string fname){
	ifstream f(fname.c_str());
	if(f.is_open()){
	    while(!f.eof()){
		string line;
		getline(f,line);
		if((*line.end())=='\r'){
		    line.erase(line.end());	/* remove the last character if it is a \r */
		}
		patterns.push_back(new myregex(line,0));
	    }
	    f.close();
	    return 0;
	}
	return -1;
    }
    /** check() is threadsafe. */
    bool check(const string &probe,std::string *found, size_t *offset,size_t *len) const {
	/* First check literals, because they are faster */
	if(literal_strings.find(probe)!=literal_strings.end()){
	    return true;
	}
	/* Now check the patterns */
	for(vector<myregex *>::const_iterator it=patterns.begin(); it != patterns.end(); it++){
	    if((*it)->search(probe,found,offset,len)){
		return true;
	    }
	}
	return false;
    }
};

#endif

