%{

#include "utf8.h"

/*
 * http://flex.sourceforge.net/manual/Cxx.html
 *
 * Credit card scanner (and then some).
 * For references, see:
 * http://en.wikipedia.org/wiki/Bank_card_number
 * http://en.wikipedia.org/wiki/List_of_Bank_Identification_Numbers
 */

#include "bulk_extractor.h"
#include "scan_ccns2.h"

#undef 	ECHO
#define	ECHO {}

#define YY_SKIP_YYWRAP
#define YY_DECL int MyAcctFlexLexer::yylex()
#define YY_INTERACTIVE 1		/* experimental */

#define MIN_PHONE_DIGITS 6

static bool has_min_digits(const char *buf,int min_digits)
{
   int digit_count = 0;
   for(const char *cc=buf;*cc;cc++){
      if(isdigit(*cc)) digit_count+=1;
   }
   return digit_count>=min_digits;
}

static string utf16to8(const wstring &s){
string utf8_line;
	try {
	    utf8::utf16to8(s.begin(),s.end(),back_inserter(utf8_line));
	} catch(utf8::invalid_utf16){
	    /* Exception thrown: bad UTF16 encoding */
	    utf8_line = "";
	}
	return utf8_line;
    }



class MyAcctFlexLexer : public acctsFlexLexer {
      public:
      class feature_recorder *ccn_recorder;
      class feature_recorder *ccn_track2;
      class feature_recorder *telephone_recorder;
      class feature_recorder *alert_recorder;

      MyAcctFlexLexer(const sbuf_t &sbuf_):sbuf(sbuf_){
	  this->pos		= 0;
          this->point		= 0;
          this->eof             = false;
      }
#include "scan_flex.inc"
};

int acctsFlexLexer::yylex(){return 0;}

%}

%option c++
%option noyywrap
%option 8bit
%option batch
%option case-insensitive
%option pointer
%option prefix="accts"
%option noyymore

END	([^0-9e\.]|(\.[^0-9]))
BLOCK    [0-9]{4}
DELIM	 ([- ])
DB	({BLOCK}{DELIM})
SDB	([45][0-9][0-9][0-9]{DELIM})
TDEL	([ /.-])

PHONETEXT ([^a-z](tel[.ephon]*|(fax)|(facsimile)|DSN|telex|TTD|mobile|cell)):?

YEAR		((19[0-9][0-9])|(200[0-9]))
DAYOFWEEK	(Mon|Tue|Wed|Thu|Fri|Sat|Sun)
DAY		([0-9]|[0-2][0-9]|30|31)
MONTH		(Jan(uary)?|Feb(ruary)?|Mar(ch)?|Apr(il)?|May|Jun(e)?|Jul(y)?|Aug(ust)?|Sep(tember)?|Oct(ober)?|Nov(ember)?|Dec(ember)?|0?1|0?2|0?3|0?4|0?5|0?6|0?7|0?8|0?9|10|11|12)
SYEAR		([0-9][0-9])
SMONTH		([0-1][0-2])

DATEA	({YEAR}-{MONTH}-{DAY})
DATEB	({YEAR}[/]{MONTH}[/]{DAY})
DATEC	({DAY}[ ]{MONTH}[ ]{YEAR})
DATED	({MONTH}[ ]{DAY}[, ]+{YEAR})

DATEFORMAT	({DATEA}|{DATEB}|{DATEC}|{DATED})

%%

[^0-9]{DB}{DB}{DB}{DB}{DB} {
    /* #### #### #### #### #### #### is definately not a CCN. */
    /* REGEX1 */
    pos += yyleng;
}

[^0-9]{SDB}{DB}{DB}{BLOCK}/{END} {
    /* REGEX2 */
    /* #### #### #### #### --- most credit card numbers*/
    /* don't include the non-numeric character in the hand-off */
    if(validate_ccn(yytext+1,yyleng-1)){
        ccn_recorder->write_buf(sbuf,pos+1,yyleng-1);
    }	
    pos += yyleng;
}

[^0-9\.][3][0-9][0-9][0-9]{DELIM}[0-9]{6}{DELIM}[0-9]{5}/{END} {
    /* 3### ###### ######### --- 15 digits beginning with 3 and funny space. */
    /* REGEX3 */
    /* Must be american express... */ 
    if(validate_ccn(yytext+1,yyleng-1)){
        ccn_recorder->write_buf(sbuf,pos+1,yyleng-1);
    }	
    pos += yyleng;
}

[^0-9\.][3]([0-9]{14})/{END} {
    /* 3### ###### ######### --- 15 digits beginning with 3 and funny space. */
    /* REGEX4 */
    /* Must be american express... */ 
    if(validate_ccn(yytext+1,yyleng-1)){
        ccn_recorder->write_buf(sbuf,pos+1,yyleng-1);
    }	
    pos += yyleng;
}

[^0-9\.][4-6]([0-9]{15,18})/{END} {
    /* REGEX5 */
    /* ###############  13-19 numbers as a block beginning with a 4 or 5
     * followed by something that is not a digit.
     * Yes, CCNs can now be up to 19 digits long. 
     * http://www.creditcards.com/credit-card-news/credit-card-appearance-1268.php
     */
    if(validate_ccn(yytext+1,yyleng-1)){
        ccn_recorder->write_buf(sbuf,pos+1,yyleng-1);
    }	
    pos += yyleng;
}

[^0-9][4-6]([0-9]{15,18})\={SYEAR}{SMONTH}101[0-9]{13} {
    /* ;###############=YYMM101#+? --- track2 credit card data */
    /* {SYEAR}{SMONTH} */
    /* ;CCN=05061010000000000738? */
    /* REGEX6 */
    if(validate_ccn(yytext+1,16)){  /* validate the first 16 digits */
    	ccn_track2->write_buf(sbuf,pos+1,yyleng-1);
    }
    pos += yyleng;
}

[^0-9][0-9][0-9][0-9]{TDEL}[0-9][0-9][0-9]{TDEL}[0-9][0-9][0-9][0-9]/{END} {
    /* REGEX7 */
    /* US phone numbers without area code in parens */
    /* New addition: If proceeded by " ####? ####? " 
     * then do not consider this a phone number. We see a lot of that stuff in
     * PDF files.
     */
    if(validate_phone(sbuf,pos+1,yyleng-1)){
       telephone_recorder->write_buf(sbuf,pos+1,yyleng-1);
    }
    pos += yyleng;
}

[^0-9]\([0-9][0-9][0-9]\){TDEL}?[0-9][0-9][0-9]{TDEL}[0-9][0-9][0-9][0-9]/{END} {
    /* REGEX8 */
    /* US phone number with parens, like (215) 555-1212 */
    telephone_recorder->write_buf(sbuf,pos+1,yyleng-1);
    pos += yyleng;
}

[^0-9]\+[0-9]{1,3}(({TDEL}[0-9][0-9][0-9]?){2,6})[0-9]{2,4}/{END} {
    /* REGEX9 */
    /* Generalized international phone numbers */
    if(has_min_digits(yytext,MIN_PHONE_DIGITS)){
        if(validate_phone(sbuf,pos+1,yyleng-1)){
            telephone_recorder->write_buf(sbuf,pos+1,yyleng-1);
        }
    }
    pos += yyleng;
}

{PHONETEXT}[0-9/ .+]{7,18} {
    /* REGEX10 */
    /* Generalized number with prefix */
    telephone_recorder->write_buf(sbuf,pos+1,yyleng);
    pos += yyleng;
}

{PHONETEXT}[0-9 +]+[ ]?[(][0-9]{2,4}[)][ ]?[\-0-9]{4,8} {
    /* REGEX11 */
    /* Generalized number with city code and prefix */
    telephone_recorder->write_buf(sbuf,pos+1,yyleng-1);
    pos += yyleng;
}

[^0-9]([0-9]{6}-){7}([0-9]{6})/[\r\n] {
    alert_recorder->write(sbuf.pos0+pos,yytext+1,"Possible BitLocker Recovery Key (ASCII).");
    pos += yyleng;
}

\0((([0-9]\0){6}-\0){7}(([0-9]\0){6}))/[^0-9] {
    /* Make a wstring from yytext+1 to yyleng */
    wstring stmp;
    for(ssize_t i=1;i+1<yyleng;i+=2){
       stmp.push_back(yytext[i] | (yytext[i+1]<<8));
    }
    alert_recorder->write(sbuf.pos0+pos,utf16to8(stmp),"Possible BitLocker Recovery Key (UTF-16)");
    pos += yyleng;
}


fedex[^a-z]+[0-9][0-9][0-9][0-9][- ]?[0-9][0-9][0-9][0-9][- ]?[0-9]/{END}	{
    /* REGEX12 */
    /* Generalized international phone numbers */
    ccn_recorder->write_buf(sbuf,pos,yyleng);
    pos += yyleng;
}

ssn:?[ \t]+[0-9][0-9][0-9]-?[0-9][0-9]-?[0-9][0-9][0-9][0-9]/{END}	{
    /* REGEX13 */
    ccn_recorder->write_buf(sbuf,pos,yyleng);
    pos += yyleng;
}

dob:?[ \t]+{DATEFORMAT}	{
    /* REGEX14 */
    ccn_recorder->write_buf(sbuf,pos,yyleng);
    pos += yyleng;
}


.|\n { 
     /**
      * The no-match rule.
      * If we are beyond the end of the margin, call it quits.
      */
     /* putchar(yytext[0]); */ /* Uncomment for debugging */
     pos++; 
     if(pos>=sbuf.pagesize) eof=true;
}
%%

extern "C"
void scan_accts(const class scanner_params &sp,const recursion_control_block &rcb)
{
    assert(sp.version==1);      
    if(sp.phase==0){
	sp.info->name  = "accts";
        sp.info->feature_names.insert("ccn");
        sp.info->feature_names.insert("ccn_track2");
        sp.info->feature_names.insert("telephone");
        sp.info->feature_names.insert(feature_recorder_set::ALERT_RECORDER);
	sp.info->histogram_defs.insert(histogram_def("ccn","","histogram"));
	sp.info->histogram_defs.insert(histogram_def("ccn_track2","","histogram"));
	sp.info->histogram_defs.insert(histogram_def("telephone","","histogram"));
	return;
    }
    if(sp.phase==2) return;

    MyAcctFlexLexer  lexer(sp.sbuf);

    lexer.ccn_recorder       = sp.fs.get_name("ccn");
    lexer.telephone_recorder = sp.fs.get_name("telephone");
    lexer.ccn_track2         = sp.fs.get_name("ccn_track2");
    lexer.alert_recorder     = sp.fs.get_name(feature_recorder_set::ALERT_RECORDER);
    lexer.run();
}
