%{

/*
 * http://flex.sourceforge.net/manual/Cxx.html
 *
 * Credit card scanner (and then some).
 * For references, see:
 * http://en.wikipedia.org/wiki/Bank_card_number
 * http://en.wikipedia.org/wiki/List_of_Bank_Identification_Numbers
 */

#include "bulk_extractor.h"

#undef 	ECHO
#define	ECHO {}

#define YY_SKIP_YYWRAP
#define YY_DECL int MyKMLScanner::yylex()
#define YY_INTERACTIVE 1		/* experimental */

class MyKMLScanner : public kmlFlexLexer {
      public:
      class feature_recorder *kml_recorder;

      MyKMLScanner(const sbuf_t &sbuf_):sbuf(sbuf_){
	  this->pos		= 0;
          this->point		= 0;
          this->eof             = false;
      }
#include "scan_flex.inc"
};

int kmlFlexLexer::yylex(){return 0;}

%}

%option c++
%option noyywrap
%option 8bit
%option batch
%option case-insensitive
%option pointer
%option prefix="kml"
%option noyymore

UNICODE		([[:print:][:space:]]+)	

%%

[<][?]xml{UNICODE}[<]kml{UNICODE}[<][/]kml[>]	{
    kml_recorder->carve(sbuf,pos,yyleng);
    pos += yyleng;
}

.|\n { 
     /**
      * The no-match rule.
      * If we are beyond the end of the margin, call it quits.
      */
     /* putchar(yytext[0]); */ /* Uncomment for debugging */
     pos++; 
     if(pos>=sbuf.pagesize) eof=true;
}
%%

extern "C"
void scan_kml(const class scanner_params &sp,const recursion_control_block &rcb)
{
    assert(sp.version==1);      
    if(sp.phase==0){
	sp.info->name = "kml";
        sp.info->feature_names.insert("kml");
	return;
    }
    if(sp.phase==2) return;
    
    if(sp.sbuf.pagesize<24) return; /* minimum size to scan */

    /* Prescan; If we do not have an 'xml' and a 'kml' in the buffer, don't run flex */
    if(sp.sbuf.find("xml",0)==-1 || sp.sbuf.find("kml",0)==-1) return ;


    MyKMLScanner  lexer(sp.sbuf);

    lexer.kml_recorder          = sp.fs.get_name("kml");
    lexer.run();
}
